'use babel';
'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _discordGraf = require('discord-graf');

var _diceExpressionEvaluator = require('dice-expression-evaluator');

var _diceExpressionEvaluator2 = _interopRequireDefault(_diceExpressionEvaluator);

var _commonTags = require('common-tags');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

Function.prototype.$asyncbind = function $asyncbind(self, catcher) {
	var resolver = this;

	if (catcher === true) {
		if (!Function.prototype.$asyncbind.EagerThenable) Function.prototype.$asyncbind.EagerThenable = function factory(tick) {
			var _tasks = [];

			if (!tick) {
				try {
					tick = process.nextTick;
				} catch (ex) {
					tick = function tick(p) {
						setTimeout(p, 0);
					};
				}
			}

			function _untask() {
				for (var i = 0; i < _tasks.length; i += 2) {
					var t = _tasks[i + 1],
					    r = _tasks[i];

					for (var j = 0; j < t.length; j++) t[j].call(null, r);
				}

				_tasks = [];
			}

			function isThenable(obj) {
				return obj && obj instanceof Object && typeof obj.then === 'function';
			}

			function EagerThenable(resolver) {
				function done(inline) {
					var w;
					if (_sync || phase < 0 || (w = _thens[phase]).length === 0) return;

					_tasks.push(result, w);

					_thens = [[], []];
					if (_tasks.length === 2) inline ? _untask() : tick(_untask);
				}

				function resolveThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 0;
					result = x;
					done(true);
				}

				function rejectThen(x) {
					if (isThenable(x)) return x.then(resolveThen, rejectThen);
					phase = 1;
					result = x;
					done(true);
				}

				function settler(resolver, rejecter) {
					_thens[0].push(resolver);

					_thens[1].push(rejecter);

					done();
				}

				function toString() {
					return 'EagerThenable{' + {
						'-1': 'pending',
						0: 'resolved',
						1: 'rejected'
					}[phase] + '}=' + result.toString();
				}

				this.then = settler;
				this.toString = toString;
				var _thens = [[], []],
				    _sync = true,
				    phase = -1,
				    result;
				resolver.call(null, resolveThen, rejectThen);
				_sync = false;
				done();
			}

			EagerThenable.resolve = function (v) {
				return isThenable(v) ? v : {
					then: function then(resolve, reject) {
						return resolve(v);
					}
				};
			};

			return EagerThenable;
		}();
		return new Function.prototype.$asyncbind.EagerThenable(boundThen);
	}

	if (catcher) {
		if (Function.prototype.$asyncbind.wrapAsyncStack) catcher = Function.prototype.$asyncbind.wrapAsyncStack(catcher);
		return then;
	}

	function then(result, error) {
		try {
			return result && result instanceof Object && typeof result.then === 'function' ? result.then(then, catcher) : resolver.call(self, result, error || catcher);
		} catch (ex) {
			return (error || catcher)(ex);
		}
	}

	function boundThen(result, error) {
		return resolver.call(self, result, error);
	}

	boundThen.then = boundThen;
	return boundThen;
};

const pattern = /^(.+?)(?:(>|<)\s*([0-9]+?))?\s*$/;

class RollDiceCommand extends _discordGraf.Command {
	constructor(bot) {
		super(bot);
		this.name = 'roll';
		this.aliases = ['dice', 'rolldice', 'diceroll', '(roll: xxxx)'];
		this.group = 'dice';
		this.groupName = 'roll';
		this.description = 'Rolls specified dice.';
		this.usage = 'roll <dice expression>';
		this.details = _commonTags.oneLine`
			Dice expressions can contain the standard representations of dice in text form (e.g. 2d20 is two 20-sided dice), with addition and subtraction allowed.
			You may also use a single greater-than (>) or less-than (<) symbol at the end of the expression to add a target - if that target is met, a success message is displayed.
			Otherwise, a failure message is shown.
		`;
		this.examples = ['roll 2d20', 'roll 3d20 - d10 + 6', 'roll d20 > 10', 'Billy McBillface attempts to slay the dragon. (Roll: d20 > 10)'];
		this.patterns = [/\(\s*(?:roll|dice|rolldice|diceroll):\s*(.+?)(?:(>|<)\s*([0-9]+?))?\s*\)/i];
	}

	run(message, args, fromPattern) {
		return new Promise(function ($return, $error) {
			if (!args[0]) return $error(new _discordGraf.CommandFormatError(this, message.server));
			try {
				const matches = fromPattern ? args : pattern.exec(args[0]);
				const dice = new _diceExpressionEvaluator2.default(matches[1]);

				// Restrict the maximum dice count
				const totalDice = dice.dice.reduce((prev, die) => prev + die.diceCount ? die.diceCount : 1, 0);
				if (totalDice > 1000) return $return({ plain: `${ message.author } might hurt themselves by rolling that many dice at once!` });

				// Roll the dice
				const rollResult = dice.roll();
				this.bot.logger.debug('Dice rolled', { dice: dice.dice, result: rollResult, totalDice: totalDice });

				// Build the list of dice
				let diceList = '';
				if (totalDice <= 100 && (rollResult.diceRaw.length > 1 || rollResult.diceRaw.length > 0 && rollResult.diceRaw[0].length > 1)) {
					diceList = rollResult.diceRaw.map((res, i) => this.bot.util.nbsp(res.length > 1 ? `${ res.join(' + ') } = ${ rollResult.diceSums[i] }` : res[0])).join(',   ');
				}

				if (matches[2]) {
					// Determine whether or not the target is met
					const target = parseInt(matches[3]);
					let success = false;
					let targetMessage;
					if (matches[2] === '>') {
						success = rollResult.roll > target;
						targetMessage = `${ !success ? 'not' : '' } greater than ${ target }`;
					} else {
						success = rollResult.roll < target;
						targetMessage = `${ !success ? 'not' : '' } less than ${ target }`;
					}

					const diceInfo = diceList ? `;   ${ diceList }` : '';
					return $return({
						plain: `${ message.author } has **${ success ? 'succeeded' : 'failed' }**. (Rolled ${ rollResult.roll }, ${ targetMessage }${ diceInfo })`,
						editable: false
					});
				} else {
					const diceInfo = diceList ? ` (${ diceList })` : '';
					return $return({
						plain: `${ message.author } rolled **${ rollResult.roll }**.${ diceInfo }`,
						editable: false
					});
				}
			} catch (err) {
				return $return({ plain: `${ message.author } specified an invalid dice expression.` });
			}
			return $return();
		}.$asyncbind(this));
	}
}
exports.default = RollDiceCommand;
//# sourceMappingURL=roll.js.map
